/* f's simple pre-mkdir checker */
/* $Date: 2004-03-23 20:19:21 +0100 (Tue, 23 Mar 2004) $ $Rev: 1005 $ */

/* lots of includes */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>

#include <sys/stat.h>

#include <unistd.h>

/* our user config settings */
#include CONFIG_H

/* include a hex dump of the config file */
#include "inc-config.h"

#if USEREGEXLIST == 1
#include <regex.h>
#endif

/* some Limits */
#define MAXPATH       	260

/* biggo function to decrement a date string */
/* only supports two formats */
int DecreaseDatedir(char *dir) {
     char *lastpart, *lastparttmp;

     lastpart = dir;
     while( (lastparttmp = strstr(lastpart,"/")) != NULL ) { lastpart = lastparttmp+1; }

     /* date dirs in MMDD format */
     if (isdigit(lastpart[0]) && isdigit(lastpart[1]) && isdigit(lastpart[2]) && isdigit(lastpart[3]) && lastpart[4] == 0) {
	  int month, day;

	  month = (lastpart[0]-'0') * 10 + (lastpart[1]-'0');
	  day = (lastpart[2]-'0') * 10 + (lastpart[3]-'0');
  
	  if (day == 1 && month > 1) {
	       int dayinmonth[] = { 0, 31,28,31,30,31,30,31,31,30,31,30,31 };
	       month--;
	       day = dayinmonth[month];
	  }
	  else if (day == 1 && month == 1) {
	       month = 12;
	       day = 31;
	  }
	  else {
	       day--;
	  }

	  sprintf(lastpart,"%02d%02d", month, day);
	  return 1;
     }
     
     /* date dirs in YYYY-MM-DD format */
     if (isdigit(lastpart[0]) && isdigit(lastpart[1]) && isdigit(lastpart[2]) && isdigit(lastpart[3]) && lastpart[4] == '-' &&
	 isdigit(lastpart[5]) && isdigit(lastpart[6]) && lastpart[7] == '-' &&
	 isdigit(lastpart[8]) && isdigit(lastpart[9]) && lastpart[10] == 0)
     {
	  int year,month,day;

	  year = (lastpart[0]-'0') * 1000 + (lastpart[1]-'0') * 100 + (lastpart[2]-'0') * 10 + (lastpart[3]-'0');
	  month = (lastpart[5]-'0') * 10 + (lastpart[6]-'0');
	  day = (lastpart[8]-'0') * 10 + (lastpart[9]-'0');

	  if (day == 1 && month > 1) {
	       int dayinmonth[] = { 0, 31,28,31,30,31,30,31,31,30,31,30,31 };
	       month--;
	       day = dayinmonth[month];
	  }
	  else if (day == 1 && month == 1) {
	       month = 12;
	       day = 31;
	       year--;
	  }
	  else {
	       day--;
	  }

	  sprintf(lastpart,"%04d-%02d-%02d", year, month, day);
	  return 1;
     }

     return 0;
}

/* main program */

int main(int argc, char *argv[]) {
     const char *cwd, *tomkdir;
     char *invnukedir, invnukedirbuff[MAXPATH];
     int mkdirlen;

     if (argc == 2 && strcasecmp(argv[1],"dumpconfig") == 0) {
	  fwrite(confighdata, 1, sizeof confighdata, stdout);
	  return 0;
     }

     if (argc < 3) {
	  printf("Insufficient arguments. I should be called by glftpd as a pre_dir_check script.\n");
	  return 0;
     }

     tomkdir = argv[1];
     cwd = argv[2];
     mkdirlen = strlen(tomkdir);
 
     /* Checks for annoying .2 .3 etc uploads */
     if (mkdirlen > 2 && tomkdir[mkdirlen-2] == '.' &&
	 (tomkdir[mkdirlen-1] >= '1' && tomkdir[mkdirlen-1] <= '9'))
     {
	  printf("This release looks like a .2 upload attempt. Stop that now!\n");
	  return 2;
     }	

     /* Prevents nukes from other sites to be uploaded */
     if (strncasecmp(tomkdir,"NUKED-",6) == 0 ||
	 strncasecmp(tomkdir,"NUKED_",6) == 0 ||
	 strncasecmp(tomkdir,"_NUKED-",7) == 0 ||
	 strncasecmp(tomkdir,"!NUKED-",7) == 0 ||
	 strncasecmp(tomkdir,"!!NUKED-",8) == 0 ||
	 strncasecmp(tomkdir,"[NUKED]-",8) == 0 ||
	 strncasecmp(tomkdir,"(NUKED)-",8) == 0 ||
	 strncasecmp(tomkdir,"[NUKED] ",8) == 0 ||
	 strncasecmp(tomkdir,"[NUKED]-",8) == 0 ||
	 strncasecmp(tomkdir,"![NUKED]-",9) == 0 ||
#ifdef NUKEPREFIX
	 strncasecmp(tomkdir,NUKEPREFIX,strlen(NUKEPREFIX)) == 0 ||
#endif
	 0)
     {
	  printf("This release name looks nuked. Think again before uploading it!\n");
	  return 2;
     }

#ifdef NUKEPREFIX
     /* Check for a nuked version of this release */

     /* Figure out the invers nukedir's name. */
     if (strncasecmp(tomkdir,NUKEPREFIX,strlen(NUKEPREFIX)) == 0) {
	  strcpy(invnukedirbuff, tomkdir);
	  invnukedir = invnukedirbuff + strlen(NUKEPREFIX);
     } else {
	  invnukedir = invnukedirbuff;
	  strcpy(invnukedir, NUKEPREFIX);
	  strcat(invnukedir, tomkdir);
     }

     {
	  struct stat stbuf;
	  char statdir[MAXPATH];

	  snprintf(statdir, sizeof statdir, "%s/%s", cwd, invnukedir);

	  if (stat(statdir,&stbuf) == 0) {
	       printf("This release was already nuked today. Think before uploading again!\n");
	       return 2;
	  }
     }
#ifdef LOOKBACKDAYS
     {
	  int dayback;
	  struct stat stbuf;
	  char statdir[MAXPATH], daydir[MAXPATH];
	  
	  strcpy(daydir, cwd);
	  for(dayback = 0; dayback < LOOKBACKDAYS; dayback++) {

	       if (!DecreaseDatedir(daydir)) {
		    /* cant decrease day dir string. prolly not a dated dir */
		    break;
	       }	

	       snprintf(statdir, sizeof statdir, "%s/%s", daydir, tomkdir);

	       if (stat(statdir,&stbuf) == 0) {
		    printf("This release has already been uploaded lately. No need for it again.\n");
		    return 2;
	       }

	       snprintf(statdir, sizeof statdir, "%s/%s", daydir, invnukedir);

	       if (stat(statdir,&stbuf) == 0) {
		    printf("This release has already been uploaded (and nuked) lately. No need for it again.\n");
		    return 2;
	       }
	  }
     }
#endif
#endif

#if USEREGEXLIST == 1
#define REGEXNUM	(sizeof regexlist / sizeof regexlist[0])
     {
	  int n, match;
	  regex_t reg;
	  regmatch_t regmatches[2];

	  for (n = 0; n < REGEXNUM; n++) {
	       
	       /* pathmatch */
	       if (regexlist[n].pathmatch) {
		    if (regcomp(&reg, regexlist[n].pathmatch, REG_EXTENDED | REG_ICASE) != 0) {
			 /* Cannot compile regex. This should be checked by the regextest */
			 continue;
		    }
       
		    if (regexec(&reg, cwd, 2, regmatches, 0) != 0) {
			 continue;
		    }
		    regfree(&reg);
	       }

	       /* pathignore */
	       if (regexlist[n].pathignore) {
		    if (regcomp(&reg, regexlist[n].pathignore, REG_EXTENDED | REG_ICASE) != 0) {
			 /* Cannot compile regex. This should be checked by the regextest */
			 continue;
		    }
       
		    if (regexec(&reg, cwd, 2, regmatches, 0) == 0) {
			 continue;
		    }
		    regfree(&reg);
	       }

	       /* file regex */
	       if (regcomp(&reg, regexlist[n].regex, REG_EXTENDED | REG_ICASE) != 0) {
		    /* Cannot compile regex. This should be checked by the regextest */
		    continue;
	       }
	       
	       match = regexec(&reg, tomkdir, 2, regmatches, 0);

	       if ( (regexlist[n].matchtype == REGEX_MUSTMATCH && match != 0) ||
		    (regexlist[n].matchtype == REGEX_NOTMATCH  && match == 0) )
	       {
		    printf("%s\n",regexlist[n].message);
		    return 2;		    
	       }

	       regfree(&reg);
	  }  
     }
#endif

     /* Nothing found */
     return 0;
}
